print("import: 'markdown_it'")
import markdown_it

print("import: 'markdown_it.cli'")
import markdown_it.cli

print("import: 'markdown_it.common'")
import markdown_it.common

print("import: 'markdown_it.helpers'")
import markdown_it.helpers

print("import: 'markdown_it.presets'")
import markdown_it.presets

print("import: 'markdown_it.rules_block'")
import markdown_it.rules_block

print("import: 'markdown_it.rules_core'")
import markdown_it.rules_core

print("import: 'markdown_it.rules_inline'")
import markdown_it.rules_inline

